/*  marshal.c -- simple esci marshalling protocol over tcp/ip
 *  Copyright (C) 2008, 2009  SEIKO EPSON Corporation
 *
 *  License: AVASYS Public License
 *  Authors: AVASYS Corporation
 *
 *  This file is part of the SANE backend distributed with Image Scan!
 *  It is distributed under the terms of the AVASYS PUBLIC LICENSE.
 *
 *  You should have received a verbatim copy of the AVASYS PUBLIC
 *  LICENSE along with the software.
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "marshal.h"
#include "message.h"

/*! guaranteed to read all the data, or return -1 if an error occurred */
static ssize_t
read_all (int fd, void *buf, size_t size)
{
  ssize_t n = 0;
  ssize_t t = 1;

  while (n < size && t > 0)
   {
     errno = 0;
     t = read (fd, buf + n, size - n);
     if (0 > t || 0 != errno)
       {
          err_major ("read failed: %s\n", strerror (errno));
          return -1;
       }
     else
       {
         n += t;
         log_call ("transferred %zd bytes, total %zd/%zd\n", t, n, size);
       }
   }

  return n;
}

/*! guaranteed to write all the data, or return -1 if an error occurred */
static ssize_t
write_all (int fd, const void *buf, size_t size)
{
  ssize_t n = 0;
  ssize_t t = 1;

  while (n < size && t > 0)
   {
     errno = 0;
     t = write (fd, buf + n, size - n);
     if (0 > t || 0 != errno)
       {
          err_major ("write failed: %s\n", strerror (errno));
          return -1;
       }
     else
       {
         n += t;
         log_call ("transferred %zd bytes, total %zd/%zd\n", t, n, size);
       }
   }

  return n;
}

ssize_t
marshal_write (int sock, uint16_t id, uint8_t type_status,
               size_t size, const void* payload)
{
  ssize_t n = 0;

  if (0 > write_all (sock, &id, sizeof (id))) return -1;
  if (0 > write_all (sock, &type_status, sizeof (type_status))) return -1;
  if (0 > write_all (sock, &size, sizeof (size))) return -1;

  if (!payload || 0 >= size) return 0;

  n = write_all (sock, payload, size);
  return n;
}

/*! Caller must have allocated space for \a id, and \a type_status.
 *  \a payload is automatically allocated based on the size field of the
 *  marshalling header, and it is the responsibility of the caller to
 *  deallocate it.
 */
ssize_t
marshal_read (int sock, uint16_t *id, uint8_t *type_status,
              void** payload)
{
  size_t size = 0;
  char* buf = NULL;

  if (0 > read_all (sock, id, sizeof (*id))) return -1;
  if (0 > read_all (sock, type_status, sizeof (*type_status))) return -1;
  if (0 > read_all (sock, &size, sizeof (size))) return -1;

  if (!payload || 0 >= size) return 0;

  buf = (char*)malloc (size);
  if (!buf) return -1;
  memset (buf, 0, size);
  
  if (0 > read_all (sock, buf, size)) return -1;

  *payload = buf;
  return size;
}
